"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

require("core-js/modules/es7.symbol.async-iterator");

require("core-js/modules/es6.symbol");

require("core-js/modules/web.dom.iterable");

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _zip2 = _interopRequireDefault(require("lodash/zip"));

var _flattenDeep2 = _interopRequireDefault(require("lodash/flattenDeep"));

var _anyShellEscape = _interopRequireDefault(require("any-shell-escape"));

var _inspectCustomSymbol = _interopRequireDefault(require("inspect-custom-symbol"));

/*
 * wrapper class for already escaped/preserved values.
 * the `shell` function extracts the escaped/preserved value
 * from instances of this class rather than escaping them again
 */
var Escaped =
/*#__PURE__*/
function () {
  function Escaped(value) {
    (0, _classCallCheck2.default)(this, Escaped);
    this.value = value;
  }

  (0, _createClass2.default)(Escaped, [{
    key: "toString",
    value: function toString() {
      return this.value;
    } // for console.log etc.

  }, {
    key: _inspectCustomSymbol.default,
    value: function value() {
      return this.value;
    }
  }]);
  return Escaped;
}();
/*
 * performs the following mappings on the supplied value(s):
 *
 * - already-escaped/preserved values are passed through verbatim
 * - null and undefined are ignored (i.e. mapped to empty arrays,
 *   which are pruned by flatten)
 * - non-strings are stringified e.g. false -> "false"
 *
 * then flattens the resulting array and returns its elements joined by a space
 */


function _shellEscape(params) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var escaped = [__shellEscape(params, options)];
  var flattened = (0, _flattenDeep2.default)(escaped);
  return flattened.join(' ');
}
/*
 * the (recursive) guts of _shellEscape. returns a leaf node (string)
 * or a possibly-empty array of arrays/leaf nodes. prunes
 * null and undefined by returning empty arrays
 */


function __shellEscape(params, options) {
  if (params instanceof Escaped) {
    return params.value;
  } else if (Array.isArray(params)) {
    return params.map(function (param) {
      return __shellEscape(param, options);
    });
  } else if (params == null) {
    return [];
  } else {
    return options.verbatim ? String(params) : (0, _anyShellEscape.default)(String(params));
  }
}
/*
 * escapes embedded string/array template parameters and passes
 * through already escaped/preserved parameters verbatim
 */


function shell(strings) {
  var result = '';

  for (var _len = arguments.length, params = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    params[_key - 1] = arguments[_key];
  }

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _zip2.default)(strings, params)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step$value = (0, _slicedToArray2.default)(_step.value, 2),
          string = _step$value[0],
          param = _step$value[1];

      result += string + _shellEscape(param);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return result;
}
/*
 * helper function which escapes its parameters and prevents them
 * from being re-escaped
 */


shell.escape = function escape() {
  for (var _len2 = arguments.length, params = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    params[_key2] = arguments[_key2];
  }

  return new Escaped(_shellEscape(params, {
    verbatim: false
  }));
};
/*
 * helper function which protects its parameters from being escaped
 */


shell.preserve = function verbatim() {
  for (var _len3 = arguments.length, params = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
    params[_key3] = arguments[_key3];
  }

  return new Escaped(_shellEscape(params, {
    verbatim: true
  }));
};

shell.protect = shell.verbatim = shell.preserve;
module.exports = shell;
module.exports.default = module.exports;